package storage

import (
	"fmt"
	"strconv"

	v2 "gitea.timerzz.com/kedaya_haitao/common/structs/v2"
	"github.com/samber/lo"
	"gorm.io/gorm"
	"gorm.io/gorm/clause"
)

type ProviderApi interface {
	Get(id uint) (provider v2.Provider, err error)
	GetByProvider(providerId v2.ProviderId) (provider v2.Provider, err error)
	List(query PageListQuery) (providers []v2.Provider, total int64, err error)
	Find(query Scoper) (providers []v2.Provider, err error)
	Create(provider v2.Provider) error
	Update(provider v2.Provider) error
	UpdateStatus(provider v2.Provider) error
	Delete(id uint) (provider v2.Provider, err error)
	AutoMigrate() error
}

func NewProviderApi(db *gorm.DB) ProviderApi {
	return &providerApi{db: db}
}

type providerApi struct {
	db *gorm.DB
}

type ListProviderQuery struct {
	ID         uint          `query:"id"`
	ProviderId v2.ProviderId `query:"providerId"`
	Keyword    string        `query:"keyword"`
}

func (g *ListProviderQuery) Scope(db *gorm.DB) *gorm.DB {
	if g.ID > 0 {
		db = db.Where("id=?", g.ID)
	}
	if g.ProviderId != "" {
		db = db.Where("provider_id=?", g.ProviderId)
	}
	if g.Keyword != "" {
		db = db.Where("name ilike ?", "%"+g.Keyword+"%")
	}
	return db
}

func (p *providerApi) Get(id uint) (provider v2.Provider, err error) {
	err = p.db.Where("id=?", id).Preload("CalculateProcess").First(&provider).Error
	return
}

func (p *providerApi) GetByProvider(providerId v2.ProviderId) (provider v2.Provider, err error) {
	err = p.db.Where("provider_id=?", providerId).Preload("CalculateProcess").First(&provider).Error
	return
}

func (p *providerApi) List(query PageListQuery) (providers []v2.Provider, total int64, err error) {
	err = p.db.Scopes(query.Scoper.Scope).Model(&v2.Provider{}).Count(&total).Error
	if err != nil {
		return
	}
	err = p.db.Scopes(query.Scope).Order("id DESC").Find(&providers).Error
	return
}

func (p *providerApi) Find(query Scoper) (providers []v2.Provider, err error) {
	err = p.db.Scopes(query.Scope).Order("id DESC").Find(&providers).Error
	return
}

func (p *providerApi) Create(provider v2.Provider) error {
	if provider.ProviderId == "" {
		return fmt.Errorf("标识为空")
	}
	if provider.Name == "" {
		return fmt.Errorf("名称为空")
	}
	return p.db.Create(&provider).Error
}

func (p *providerApi) Update(provider v2.Provider) error {
	if provider.ProviderId == "" {
		return fmt.Errorf("标识为空")
	}

	return p.db.Transaction(func(tx *gorm.DB) error {
		ids := lo.Map(lo.Filter(provider.CalculateProcess, func(item v2.CalculateProcess, index int) bool { return item.ID > 0 }), func(item v2.CalculateProcess, index int) string {
			return strconv.Itoa(int(item.ID))
		})

		// 删除已经没有的CalculateProcess
		if err := tx.Not(ids).Where("owner_id=? AND kind=?", provider.ID, "provider").Delete(&v2.CalculateProcess{}).Error; err != nil {
			return err
		}

		// 更新方案的name和描述
		return tx.Session(&gorm.Session{FullSaveAssociations: true}).Select("updated_at", "name", "config", "mark", "CalculateProcess").
			Where("id=?", provider.ID).Updates(&provider).Error
	})
}

func (p *providerApi) UpdateStatus(provider v2.Provider) error {
	if provider.ID == 0 {
		return fmt.Errorf("id为空")
	}
	return p.db.Where("id=?", provider.ID).Select("status", "msg", "pull_at").Updates(&provider).Error
}

func (p *providerApi) Delete(id uint) (provider v2.Provider, err error) {
	err = p.db.Transaction(func(tx *gorm.DB) error {
		if err = tx.Where("owner_id=? AND kind=?", id, "provider").Delete(&v2.CalculateProcess{}).Error; err != nil {

		}
		return p.db.Clauses(clause.Returning{}).Where("id=?", id).Delete(&provider).Error
	})
	return
}

func (p *providerApi) AutoMigrate() error {
	return p.db.AutoMigrate(&v2.Provider{}, &v2.CalculateProcess{})
}
